/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



// This handles the update of the main window
#include "stdafx.h"
#include "TinyCad.h"
#include "TinyCadView.h"
#include "colour.h"
#include "revision.h"
#include "option.h"
#include "ruler.h"


/////////////////////////////////////////////////////////////////////////////
// CTinyCadView drawing

void CTinyCadView::OnDraw(CDC* pDC)
{
	CTinyCadDoc* pDoc = GetDocument();

  int selected;
  Context dc(pDC,theTransform,this);


  if (pDC->IsPrinting())
  {
	  dc.SetBlack(((CTinyCadApp*)AfxGetApp())->GetPrintBandW());
  }


  CPoint Start,End;
  CRect rect;
  GetClientRect( &rect );
  TransformSnap snap;
  snap.SetGridSnap( FALSE );
  Start=theTransform.DeScale(snap,CPoint(rect.left,rect.top));
  End=theTransform.DeScale(snap,CPoint(rect.right,rect.bottom));


  // Is any of this region in the off-page area?
  if (!pDC->IsPrinting())
  {

	  // Paint the region white
	  dc.SelectBrush(cWHITE);
	  dc.SelectPen(PS_SOLID,1,cWHITE);
	  // dc.Rectangle(CRect(Start.x,Start.y,End.x,End.y));

	  dc.SelectBrush(cOFFPAGE);
	  dc.SelectPen(PS_SOLID,1,cOFFPAGE);

	  if (End.x>GetDocument()->GetPageBoundries().x) {
		CPoint a=CPoint(GetDocument()->GetPageBoundries().x,0);
		dc.Rectangle(CRect(a.x,a.y,End.x,End.y));
	  }
	  if (End.y>GetDocument()->GetPageBoundries().y) {
		CPoint a=CPoint(Start.x,GetDocument()->GetPageBoundries().y);
		dc.Rectangle(CRect(a.x,a.y,End.x,End.y));
	  }
	  if (Start.x<0)
		dc.Rectangle(CRect(0,Start.y,Start.x,End.y));
	  if (Start.y<0)
		dc.Rectangle(CRect(Start.x,0,End.x,Start.y));

	  // Fill this region with a grid
	  int grid = GetDocument()->m_snap.GetGrid();
	  int SGrid = dc.theTransform.Scale(grid);
	  if (GetDocument()->GetOptions()->ShowGrid() && SGrid>5) {
		int xi,yi;
		int x = dc.theTransform.GetOrigin().x;
		int y = dc.theTransform.GetOrigin().y;

		BOOL doall = SGrid>10, showX = (GetDocument()->m_snap.GetAccurateGrid()==FineGrid);

		TransformSnap s = GetDocument()->m_snap;
		s.SetGridSnap(TRUE);

   		for (int xp=x>=0 ? x : 0;xp<End.x && xp<GetDocument()->GetPageBoundries().x;xp+=grid) {
			for (int yp=y>=0 ? y : 0;yp<End.y && yp<GetDocument()->GetPageBoundries().y;yp+=grid) {
				xi = s.Snap(xp);
				yi = s.Snap(yp);
				if ((xi%(NormalGrid / 10)) == 0 && (yi%(NormalGrid/10)) == 0) {
					dc.SetPixel(CPoint(xi,yi),0);
					if (showX) {
						dc.SetPixel(CPoint(xi+1,yi),0);
						dc.SetPixel(CPoint(xi-1,yi),0);
						dc.SetPixel(CPoint(xi,yi+1),0);
						dc.SetPixel(CPoint(xi,yi-1),0);
					}
				} else if (doall)
					dc.SetPixel(CPoint(xi,yi),0);
			}
		}
	  }
  }



  Start -= CPoint(10,10);
  End += CPoint(10,10);


	GetDocument()->GetSelectBegin();

	drawingIterator it = GetDocument()->GetDrawingBegin();
	while (it != GetDocument()->GetDrawingEnd()) 
	{
		CDrawingObject *obj = *it;

		selected=!pDC->IsPrinting() && GetDocument()->IsSelected( obj );
		paint_options options = selected ? draw_selected : draw_normal;

		if (!pDC->IsPrinting() || !obj->IsConstruction())
		{
			if (pDC->IsPrinting() || obj->IsInside(Start.x, End.x, Start.y, End.y ))
			{
				obj->Paint(dc,options);
			}
		}

		++ it;
	}
 

  // Now draw the selectable object, so it stands out...
  CDrawingObject *obj = GetDocument()->GetSelectable();
  if ( obj != NULL && !GetDocument()->IsSelected( obj ))
  {
	  paint_options options = draw_selectable;
	  GetDocument()->GetSelectable()->Paint( dc, options );
  }

  // If only one item is selected then just draw its handles
  // now...
  if (GetDocument()->IsSingleItemSelected())
  {
	  GetDocument()->GetSingleSelectedItem()->PaintHandles( dc );
  }

  // if necessary turn back on the current object to be edited
  if (GetDocument()->GetEdit()!= NULL)
	GetDocument()->GetEdit()->Paint(dc,draw_selected);

  // Draw the design details
  GetDocument()->DisplayDetails(dc);

  // Draw the design rulers
  GetDocument()->DisplayRulers(dc);

}



// The Zoom function
// OnSetZoom:
// Sets the zoom from a menu selection
void CTinyCadView::ChangeZoom(LONG NewZoom)
{
  theTransform.SetZoom(NewZoom);
}


void CTinyCadView::SetZoom(LONG NewZoom)
{
  ChangeZoom(NewZoom);
  SetScroll(theTransform.GetOrigin().x,theTransform.GetOrigin().y);
  Invalidate();
}



// Track the size of the window
void CTinyCadView::OnSize(UINT a,int cx,int cy)
{
	CView::OnSize(a,cx,cy);

	if (IsWindowVisible())
	{
		((CTinyCadApp*)AfxGetApp())->SetMdiMaximize( GetParent()->IsZoomed() );	
	}


	if (m_pDocument == NULL)
		return;

	CRect nSize;
	GetClientRect(nSize);
	if (vRuler != NULL)
		vRuler->OnNewSize(nSize);
	if (hRuler != NULL)
		hRuler->OnNewSize(nSize);

	SetScroll(theTransform.GetOrigin().x,theTransform.GetOrigin().y);

}


// The scroll bar functions:
// These control the horizontal and vertical scrolling of the window
// (uses OnSize to track the window's size to stop the user being able to place text off the max size)
static LONG oldZoom = 100;
void CTinyCadView::SetScroll(int NewX,int NewY)
{
  CRect rect;
  int px,py;

  // How big is the current window?
  GetClientRect(rect);

  // Convert region into internal co-ords
  px=theTransform.intDeScale(rect.right);
  py=theTransform.intDeScale(rect.bottom);


  // Allow a 10% overlap
  int Xlap=GetDocument()->GetPageBoundries().x/10;
  int Ylap=GetDocument()->GetPageBoundries().y/10;

  // Now does this fit?
  if (px+NewX>GetDocument()->GetPageBoundries().x+Xlap)
	NewX=GetDocument()->GetPageBoundries().x+Xlap-px;
  if (py+NewY>GetDocument()->GetPageBoundries().y+Ylap)
	NewY=GetDocument()->GetPageBoundries().y+Ylap-py;
  if (NewX<-Xlap)
	NewX=-Xlap;
  if (NewY<-Ylap)
	NewY=-Ylap;

  if (theTransform.GetOrigin()!=CPoint(NewX,NewY) || oldZoom!=theTransform.GetZoom()) {
  	theTransform.SetOriginX(NewX);
  	theTransform.SetOriginY(NewY);
	SetScrollRange(SB_HORZ,-Xlap,GetDocument()->GetPageBoundries().x+Xlap-px,FALSE);
	SetScrollRange(SB_VERT,-Ylap,GetDocument()->GetPageBoundries().y+Ylap-py,FALSE);
        SetScrollPos(SB_HORZ,theTransform.GetOrigin().x);
	SetScrollPos(SB_VERT,theTransform.GetOrigin().y);

	if (hRuler != NULL)
		hRuler->OnNewTransform(theTransform);
	if (vRuler != NULL)
		vRuler->OnNewTransform(theTransform);
	Invalidate();
  }

  oldZoom=theTransform.GetZoom();
}

void CTinyCadView::SetScrollCentre(CPoint c)
{
  CRect rect;
  CPoint p;

  GetClientRect(rect);
  p=theTransform.DeScale(GetDocument()->m_snap,CPoint(rect.right/2,rect.bottom/2));
  SetScroll(theTransform.GetOrigin().x+c.x-p.x,theTransform.GetOrigin().y+c.y-p.y);  
}



// The message handlers for the VScroll and HScroll messages
void CTinyCadView::OnHScroll( UINT wParam, UINT pos, CScrollBar* )
{
  CRect rect;
  int moveFast,moveSlow;
  int x = theTransform.GetOrigin().x;
  int y = theTransform.GetOrigin().y;

  GetClientRect(rect);
  int Width = theTransform.intDeScale(rect.right);
  moveFast=(Width<<4)/20;
  moveSlow=(Width*GetDocument()->GetOptions()->GetScrollBar())/100;

  switch (wParam) {
	case SB_LINEUP:
		SetScroll(x-moveSlow,y);
		break;
	case SB_PAGEUP:
		SetScroll(x-moveFast,y);
		break;
	case SB_LINEDOWN:
		SetScroll(x+moveSlow,y);
		break;
	case SB_PAGEDOWN:
		SetScroll(x+moveFast,y);
		break;
	case SB_THUMBPOSITION:
		SetScroll(pos,y);
		break;
  }
}


// The message handlers for the VScroll and HScroll messages
void CTinyCadView::OnVScroll( UINT wParam, UINT pos, CScrollBar* )
{
  CRect rect;
  int moveFast,moveSlow;
  int x = theTransform.GetOrigin().x;
  int y = theTransform.GetOrigin().y;


  GetClientRect(rect);
  int Width = theTransform.intDeScale(rect.bottom);
  moveFast=(Width<<4)/20;
  moveSlow=(Width*GetDocument()->GetOptions()->GetScrollBar())/100;

  switch (wParam) {
	case SB_LINEUP:
		SetScroll(x,y-moveSlow);
		break;
	case SB_PAGEUP:
		SetScroll(x,y-moveFast);
		break;
	case SB_LINEDOWN:
		SetScroll(x,y+moveSlow);
		break;
	case SB_PAGEDOWN:
		SetScroll(x,y+moveFast);
		break;
	case SB_THUMBPOSITION:
		SetScroll(x,pos);
		break;
  }
}


