/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "TinyCadView.h"


////// The Symbol Field class //////

// Save this symbol field into an archive
void CSymbolField::Save(CStream &ar)
{
	DWORD ft = field_type;
	ar << field_name << field_value << field_default << field_pos << ft;
}

// Load this symbol field from an archive
void CSymbolField::Load(CStream &ar)
{
	DWORD ft;
	ar >> field_name >> field_value >> field_default >> field_pos >> ft;
	field_type = (SymbolFieldType)ft;
}



////// The Symbol Record class //////

// The constructor
CSymbolRecord::CSymbolRecord()
{
	name_type = default_show;
	ref_type = default_show;
	ppp = 1;
    fields_loaded = FALSE;
	FilePos = -1;
	orientation = 0; // Default to Up!
}


BOOL CSymbolRecord::LoadSymbol( CTinyCadDoc *pDesign, CStream &theArchive, CPoint &tr, drawingCollection &drawing )
{
  // Move to the right location in filespace
  try
  {
	  // Load in the design
	  if (!pDesign->ReadFile( theArchive, FALSE, drawing ))
	  {
		  return FALSE;
	  }
  }
  catch (...)
  {
	  // Failed to load the symbol
	  return FALSE;
  }

  // Get the default orientation
  orientation = theArchive.GetOrientation();

  // Find the co-ords of the bounding box of this symbol
  CPoint a=CPoint(0,0);
  CPoint b=drawing.front()->a;

  BOOL has_pins = FALSE;
  CPoint pin = CPoint(0,0);

  drawingIterator it = drawing.begin();
  while (it != drawing.end()) 
  {
	  CDrawingObject *pointer = *it;

	if (!has_pins && pointer->GetType()==xPinEx)
	{
		CDrawPin *thePin = static_cast<CDrawPin*>(pointer);
		if (!thePin->IsInvisible())
		{
			// Use this for the pin-offset calculation
			pin = pointer->a;
			has_pins = TRUE;
		}
	}


	a.x=max(a.x,max(pointer->a.x,pointer->b.x));
	a.y=max(a.y,max(pointer->a.y,pointer->b.y));

	b.x=min(b.x,min(pointer->a.x,pointer->b.x));
	b.y=min(b.y,min(pointer->a.y,pointer->b.y));
	
	++ it;
  }

  // If this symbol has pins, then snap the pin to the grid,
  // rather than the bounding rect
  CPoint snapa, snapb;
  CPoint pin_offset = CPoint(0,0);

  // Snap the co-ords of the bounding box to the grid
  snapa = pDesign->m_snap.Snap(a);
  snapb = pDesign->m_snap.Snap(b);

  // Now make sure the pins stay on-grid
  if (has_pins)
  {
	  // Snap the co-ords of the pin to the grid
	  pin_offset=pin - pDesign->m_snap.Snap(pin);

	  snapa = snapa + pin_offset;
	  snapb = snapb + pin_offset;
  }

  
  // Make sure snapping is always positive
  if (snapa.x<a.x)
	snapa.x+=pDesign->m_snap.GetGrid();
  if (snapa.y<a.y)
	snapa.y+=pDesign->m_snap.GetGrid();
  if (snapb.x>b.x)
	snapb.x-=pDesign->m_snap.GetGrid();
  if (snapb.y>b.y)
	snapb.y-=pDesign->m_snap.GetGrid();

  // Now translate so that bottom left hand corner is at 0,0
  it = drawing.begin();
  while (it != drawing.end()) 
  {
	  CDrawingObject *pointer = *it;
	pointer->a=CPoint(pointer->a.x-snapa.x,pointer->a.y-snapa.y);
	pointer->b=CPoint(pointer->b.x-snapa.x,pointer->b.y-snapa.y);
	
	++ it;
  }

  // Store top right hand corner
  tr=CPoint(snapb.x-snapa.x ,snapb.y-snapa.y );

  return TRUE;
}




////// The Symbol class //////

// The constructor
CLibraryStoreSymbol::CLibraryStoreSymbol(CLibraryStore *NewLib)
{
  lib=NewLib;
  NameID = -1;
  FilePos = -1;
}

// The destructor
CLibraryStoreSymbol::~CLibraryStoreSymbol()
{
}

CString CLibraryStoreSymbol::GetLibName()
{
  return lib->m_name;
}



// Is this symbol correct filled in?
BOOL CLibraryStoreSymbol::IsValid()
{
	return name!="" && reference!="" && ppp>0;
}


// Does this string match this symbol
CString CLibraryStoreSymbol::Find(const char *theSearchString)
{
  CString HoldString;

  HoldString = name + "  " + description;
  HoldString.MakeLower();

  // Now look for the search string in this string
  return (HoldString.Find(theSearchString) != -1) ? name + " " + description : "";
}


// Save this symbol into an archive
void CLibraryStoreSymbol::Save(CStream &theArchive)
{
  DWORD flags = name_type << 3 | ref_type;
  DWORD number_of_fields = fields.size() | (flags << 16);
  theArchive << name << description << reference << ppp << number_of_fields; 

  for (int i = 0; i < fields.size(); i++)
  {
	  fields[i].Save( theArchive );
  }
}


// load this symbol from an archive
void CLibraryStoreSymbol::Load(CStream &theArchive)
{
  DWORD number_of_fields;
  FilePos = 0;
  theArchive >> name >> description >> reference >> ppp >> number_of_fields; 
  DWORD flags = number_of_fields >> 16;
  number_of_fields = number_of_fields & 0xffff;
  for (int i = 0; i < number_of_fields; i++)
  {
	  CSymbolField sf;
	  sf.Load( theArchive );
	  fields.push_back( sf );
  }

  name_type = static_cast<SymbolFieldType>(flags >> 3);
  ref_type = static_cast<SymbolFieldType>(flags & 0x7);
}

// load this symbol from an archive
void CLibraryStoreSymbol::OldLoad3(CStream &theArchive)
{
	CString dummy;
  theArchive >> name >> description >> reference >> ppp >> FilePos >> dummy; 
}


// load this symbol from an archive
void CLibraryStoreSymbol::OldLoad1(CStream &theArchive)
{
  theArchive >> name >> description >> reference >> ppp >> FilePos; 
}


// the old version of load this symbol from an archive
void CLibraryStoreSymbol::OldLoad2(CStream &theArchive)
{
  theArchive >> name >> reference >> ppp >> FilePos; 
  description = "";
}



// Get the object this symbol refers to, this method, may not be in memory
// and so will have to be extracted from the library file.
BOOL CLibraryStoreSymbol::GetMethod( CTinyCadDoc *pDesign, CPoint &tr, drawingCollection &drawing )
{
  // No, so must load it from the methods file
  CStream *stream=lib->GetMethodArchive( this );
  if (!stream)
  {
	  return FALSE;
  }
  
  BOOL r = LoadSymbol( pDesign, *stream, tr, drawing );

  delete stream;
  return r;
}



CDesignFileSymbol *CLibraryStoreSymbol::GetDesignSymbol( CTinyCadDoc *pDesign )
{
	CPoint tr;

	drawingCollection drawing;
	if (!GetMethod( pDesign, tr, drawing ))
	{
		return NULL;
	}

	CDesignFileSymbol *psymbol = new CDesignFileSymbol( pDesign, drawing, tr );

	psymbol->description = description;
	psymbol->name = name;
	psymbol->ppp = ppp;
	psymbol->reference = reference;
	psymbol->name_type = name_type;
	psymbol->ref_type = ref_type;
	psymbol->fields = fields;
	psymbol->FilePos = FilePos;

	return psymbol;
}



///// The symbol as contained by the design /////

CDesignFileSymbol::CDesignFileSymbol(CTinyCadDoc *pDesign, drawingCollection &method, CPoint tr )
{
	m_pDesign = pDesign;
	m_method = method;
	m_Tr = tr;
}

// The Destructor
CDesignFileSymbol::~CDesignFileSymbol()
{
	drawingIterator it = m_method.begin();
	while (it != m_method.end()) 
	{
		delete *it;
		++ it;
	}
}
	
// Save this symbol into an archive
void CDesignFileSymbol::Save(CStream &theArchive)
{
	DWORD flags = name_type << 3 | ref_type;
    DWORD number_of_fields = fields.size() | (flags << 16);
	CString dummy;
	dummy.Format("%d", number_of_fields );
	theArchive << name << description << reference << ppp << dummy; 
	for (int i = 0; i < fields.size(); i++)
	{
		fields[i].Save( theArchive );
	}

	m_pDesign->Save(theArchive, m_method, FALSE,FALSE,FALSE);
}

// Load this symbol from an archive
void CDesignFileSymbol::Load(CTinyCadDoc *pDesign, CStream &theArchive)
{
	CString dummy;
	theArchive >> name >> description >> reference >> ppp >> dummy; 

	DWORD number_of_fields = 0;
	if (!dummy.IsEmpty())
	{
		number_of_fields = atoi( dummy );
	}

    DWORD flags = number_of_fields >> 16;
    number_of_fields = number_of_fields & 0xffff;

	for (int i = 0; i < number_of_fields; i++)
	{
	  CSymbolField sf;
	  sf.Load( theArchive );
	  fields.push_back( sf );
	}

	name_type = static_cast<SymbolFieldType>(flags >> 3);
	ref_type = static_cast<SymbolFieldType>(flags & 0x7);


	m_pDesign = pDesign;
	LoadSymbol( pDesign, theArchive, m_Tr, m_method );
}


// Return the methods object
BOOL CDesignFileSymbol::GetMethod( CPoint &tr, drawingCollection &drawing )
{
	tr = m_Tr;
	drawing = m_method;

	return TRUE;
}
